/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { IdentifiableComponentInterface } from "@wso2is/core/models";
import { Field, Form } from "@wso2is/form";
import { Code, GenericIcon, Heading } from "@wso2is/react-components";
import isEmpty from "lodash-es/isEmpty";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { Trans, useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { Divider } from "semantic-ui-react";
import { AppState } from "../../../../../features/core/store";
import { BrandingPreferencesConstants } from "../../constants";
import { BrandingPreferenceMeta } from "../../meta";
import { BrandingPreferenceInterface } from "../../models";

/**
 * Interface for Branding Preference Design Form props.
 */
interface DesignFormPropsInterface extends IdentifiableComponentInterface {
    /**
     * Broadcast realtime edited values to subscribers.
     * @param {DesignFormValuesInterface} - Form values.
     */
    broadcastValues: (values: DesignFormValuesInterface) => void;
    /**
     * Design form initial values.
     */
    initialValues: DesignFormValuesInterface;
    /**
     * Callback for form submit.
     * @param {DesignFormValuesInterface} values - Resolved Form Values.
     */
    onSubmit: (values: DesignFormValuesInterface) => void;
    /**
     * Is readonly.
     */
    readOnly?: boolean;
    /**
     * Specifies if the form is submitting.
     */
    isSubmitting?: boolean;
}

/**
 * Form initial values interface.
 */
export type DesignFormValuesInterface = Pick<BrandingPreferenceInterface, "images">
    & Pick<BrandingPreferenceInterface, "colors">;

/**
 * Branding Preference Design Form.
 *
 * @param {DesignFormPropsInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const DesignForm: FunctionComponent<DesignFormPropsInterface> = (
    props: DesignFormPropsInterface
): ReactElement => {

    const {
        broadcastValues,
        initialValues,
        onSubmit,
        readOnly,
        isSubmitting,
        [ "data-componentid" ]: componentId
    } = props;

    const { t } = useTranslation();

    const productName: string = useSelector((state: AppState) => state.config.ui.productName);

    const [ faviconImageURL, setFaviconImageURL ] = useState<string>(initialValues.images.favicon.imgURL);
    const [ logoImageURL, setLogoImageURL ] = useState<string>(initialValues.images.logo.imgURL);
    const [ logoImageAltText, setLogoImageAltText ] = useState<string>(initialValues.images.logo.altText);
    const [ themeColors, setThemeColors ] = useState<BrandingPreferenceInterface[ "colors" ]>(initialValues.colors);

    /**
     * Broadcast values to the outside when internals change.
     */
    useEffect(() => {
        
        broadcastValues({
            ...initialValues,
            colors: {
                ...initialValues.colors,
                ...themeColors
            },
            images: {
                ...initialValues.images,
                favicon: {
                    imgURL: faviconImageURL
                },
                logo: {
                    ...initialValues.images.logo,
                    altText: logoImageAltText,
                    imgURL: logoImageURL
                }
            }
        });
    }, [ faviconImageURL, logoImageURL, logoImageAltText, themeColors ]);

    return (
        <Form
            uncontrolledForm={ false }
            onSubmit={ (values: Record<string, any>) => {
                onSubmit(values as DesignFormValuesInterface);
            } }
            enableReinitialize={ true }
            initialValues={ initialValues }
        >
            <Heading as="h4">
                { t("extensions:develop.branding.forms.design.images.heading") }
            </Heading>
            
            <Heading as="h5">
                { t("extensions:develop.branding.forms.design.images.logo.heading") }
            </Heading>
            
            <div className="image-preview">
                <GenericIcon
                    transparent
                    size="tiny"
                    icon={
                        !isEmpty(logoImageURL)
                            ? logoImageURL
                            : BrandingPreferenceMeta.getBrandingPreferenceFallbacks().images.logo.imgURL
                    }
                />
            </div>
            <Field.Input
                ariaLabel="Branding preference logo URL"
                inputType="url"
                name="images.logo.imgURL"
                label={ t("extensions:develop.branding.forms.design.images.logo.fields.url.label") }
                placeholder={ t("extensions:develop.branding.forms.design.images.logo.fields.url.placeholder") }
                hint={ (
                    <Trans
                        i18nKey={ "extensions:develop.branding.forms.design.images.logo.fields.url.hint" }
                        tOptions={ {
                            productName
                        } }
                    >
                        Use an image that’s atleast <Code>600x600 pixels</Code> and less than
                        <Code>1mb</Code> in size for better performance. If not set,
                        { productName } defaults will be used.
                    </Trans>
                ) }
                required={ false }
                value={ initialValues.images.logo.imgURL }
                readOnly={ readOnly }
                maxLength={ BrandingPreferencesConstants.DESIGN_FORM_FIELD_CONSTRAINTS.LOGO_URL_MAX_LENGTH }
                minLength={ BrandingPreferencesConstants.DESIGN_FORM_FIELD_CONSTRAINTS.LOGO_URL_MIN_LENGTH }
                width={ 16 }
                listen={ (value: string) =>  setLogoImageURL(value) }
                data-componentid={ `${ componentId }-logo-url` }
            />
            <Field.Input
                ariaLabel="Branding preference logo alt text"
                inputType="default"
                name="images.logo.altText"
                label={ t("extensions:develop.branding.forms.design.images.logo.fields.alt.label") }
                placeholder={ t("extensions:develop.branding.forms.design.images.logo.fields.alt.placeholder") }
                hint={
                    t("extensions:develop.branding.forms.design.images.logo.fields.alt.hint", { productName })
                }
                required={ false }
                value={ initialValues.images.logo.altText }
                readOnly={ readOnly }
                maxLength={ BrandingPreferencesConstants.DESIGN_FORM_FIELD_CONSTRAINTS.LOGO_ALT_TEXT_MAX_LENGTH }
                minLength={ BrandingPreferencesConstants.DESIGN_FORM_FIELD_CONSTRAINTS.LOGO_ALT_TEXT_MIN_LENGTH }
                width={ 16 }
                listen={ (value: string) =>  setLogoImageAltText(value) }
                data-componentid={ `${ componentId }-logo-alt-text` }
            />

            <Heading as="h5">
                { t("extensions:develop.branding.forms.design.images.favicon.heading") }
            </Heading>
            <div className="image-preview">
                <GenericIcon
                    square
                    transparent
                    size="x22"
                    icon={
                        !isEmpty(faviconImageURL)
                            ? faviconImageURL
                            : BrandingPreferenceMeta.getBrandingPreferenceFallbacks().images.favicon.imgURL
                    }
                />
            </div>
            <Field.Input
                ariaLabel="Branding preference favicon URL"
                inputType="url"
                name="images.favicon.imgURL"
                label={ t("extensions:develop.branding.forms.design.images.favicon.fields.url.label") }
                placeholder={ t("extensions:develop.branding.forms.design.images.favicon.fields.url.placeholder") }
                hint={ (
                    <Trans
                        i18nKey={ "extensions:develop.branding.forms.design.images.favicon.fields.url.hint" }
                        tOptions={ {
                            productName
                        } }
                    >
                        Use an image that’s atleast <Code>16x16 pixels</Code> or more with square pixel 
                        aspect ratio for better results. If not set, { productName } defaults will be used.
                    </Trans>
                ) }
                required={ false }
                value={ initialValues.images.favicon.imgURL }
                readOnly={ readOnly }
                maxLength={ BrandingPreferencesConstants.DESIGN_FORM_FIELD_CONSTRAINTS.FAVICON_URL_MAX_LENGTH }
                minLength={ BrandingPreferencesConstants.DESIGN_FORM_FIELD_CONSTRAINTS.FAVICON_URL_MIN_LENGTH }
                width={ 16 }
                listen={ (value: string) =>  setFaviconImageURL(value) }
                data-componentid={ `${ componentId }-favicon-url` }
            />

            <Divider className="compact" />
            
            <Heading as="h4">{ t("extensions:develop.branding.forms.design.colors.heading") }</Heading>
            
            <Field.ColorPicker
                ariaLabel="Primary color input field"
                label={ t("extensions:develop.branding.forms.design.colors.fields.primaryColor.label") }
                name="colors.primary"
                value={ initialValues.colors.primary }
                data-componentid={ `${ componentId }-primary-color` }
                required={ false }
                readOnly={ readOnly }
                listen={ (value: string) =>  setThemeColors({ 
                    ...themeColors,
                    primary: value
                }) }
                hint={ t("extensions:develop.branding.forms.design.colors.fields.primaryColor.hint") }
                placeholder={ t("extensions:develop.branding.forms.design.colors.fields.primaryColor.placeholder") }
            />

            <Field.Button
                size="small"
                buttonType="primary_btn"
                ariaLabel="Branding preference design form update button"
                name="update-button"
                data-componentid={ `${ componentId }-submit-button` }
                disabled={ isSubmitting }
                loading={ isSubmitting }
                label={ t("common:save") }
                hidden={ readOnly }
            />
        </Form>
    );
};

/**
 * Default props for the component.
 */
DesignForm.defaultProps = {
    "data-componentid": "design-form"
};
