/**
 * Copyright (c) 2022, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { GenericIcon, GenericIconProps, Text } from "@wso2is/react-components";
import classNames from "classnames";
import React, { CSSProperties, FC, ReactElement } from "react";
import { useTranslation } from "react-i18next";
import { Card } from "semantic-ui-react";

export type CategoryItemPropsInterface = {
    categoryName: string;
    onClick: (event: React.SyntheticEvent) => void;
    comingSoon?: boolean;
    techStackIcons?: [
        GenericIconProps["icon"],
        GenericIconProps["icon"]?,
        GenericIconProps["icon"]?
    ];
    iconSize?: GenericIconProps["size"];
    loading?: boolean;
};

/**
 * A category item denotes top level application template type.
 * @param props {CategoryItemPropsInterface}
 * @constructor
 */
export const CategoryItem: FC<CategoryItemPropsInterface> = (props): ReactElement => {

    const {
        categoryName,
        iconSize,
        techStackIcons,
        onClick,
        comingSoon
    } = props;

    const { t } = useTranslation();

    const classes = classNames(
        "selection-card",
        "with-image",
        "px-3",
        {
            disabled: comingSoon,
            grayscale: comingSoon
        }
    );

    const cardStyles = (): CSSProperties => ({
        borderRadius: "8px",
        minHeight: "140px"
    });

    const technologyLogoRowStyles = (): CSSProperties => ({
        display: "flex",
        flexDirection: "row",
        gap: "2px"
    });

    const cardContentStyles = (): CSSProperties => ({
        alignItems: "center",
        display: "flex",
        flexDirection: "column",
        justifyContent: "center",
        position: "relative",
        textAlign: "center"
    });

    return (
        <Card
            data-componentid={ `application-integration-${ categoryName }` }
            data-testid={ `application-integration-${ categoryName }` }
            as="div"
            onClick={ (event: React.SyntheticEvent) => {
                event?.preventDefault();
                onClick(event);
            } }
            className={ classes }
            style={ cardStyles() }>
            <Card.Content
                className="card-image-container"
                style={ cardContentStyles() }>
                { comingSoon ? (
                    <div className="coming-soon-ribbon">
                        { t("common:comingSoon") }
                    </div>
                ) : null }
                <div style={ technologyLogoRowStyles() }>
                    { techStackIcons.map((icon, index) => {
                        const uniqueId = `${ categoryName }-tech-stack-icon-${ index }`;

                        return (
                            <GenericIcon
                                disabled={ comingSoon }
                                key={ uniqueId }
                                data-componentid={ uniqueId }
                                data-testid={ uniqueId }
                                className="card-image"
                                size={ iconSize }
                                icon={ icon }
                                square
                                transparent
                            />
                        );
                    }) }
                </div>
            </Card.Content>
            <Card.Description>
                <Text
                    muted={ comingSoon }
                    className="text-center">
                    { categoryName }
                </Text>
            </Card.Description>
        </Card>
    );

};

/**
 * Default props of {@link CategoryItem}
 */
CategoryItem.defaultProps = {
    comingSoon: false,
    iconSize: "mini",
    loading: false,
    techStackIcons: undefined
};
